/*
    myrrdin, a 2d tile engine
    Copyright (c) 2008 Friedrich Weber

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
 */

module myrrdin.director;

import myrrdin.renderer;
import myrrdin.scenes.scene;
import myrrdin.consumers.consumer;
import myrrdin.imagecache;

class DirectorClass {
    public Renderer renderer;
    private Scene current_scene = null;
    public Cache cache;

    this(char[] gfx_dir, char[] title, int width, int height, int depth=32) {
	this.renderer = new Renderer(title, width, height, depth);	
	this.cache = new Cache(gfx_dir);
    }

    void run() {
	this.renderer.mainloop();
    }

    void set_scene(Scene scene) {
	if(this.current_scene) {
	    this.unset_scene();    
	}
	this.current_scene = scene;
	this.renderer.draw_consumers ~= scene.draw_consumers;
	this.renderer.event_consumers ~= scene.event_consumers;
    }

    protected void unset_scene() {
	foreach(Consumer c; this.renderer.draw_consumers) {
	    this.renderer.remove_consumer(c);
	}
	foreach(Consumer c; this.renderer.event_consumers) {
	    this.renderer.remove_consumer(c);
	}
    }
}

void CreateDirector(char[] gfx_dir, char[] title, int width, int height, int depth=32) {
    assert(Director is null, "You can set only one director.");
    Director = new DirectorClass(gfx_dir, title, width, height, depth);
}

DirectorClass Director = null;
